@@ -18,10 +18,9 @@ namespace uhd { namespace usrp { namespace x400 {
  */
 struct adc_self_cal_params_t
 {
-    double min_gain;
-    double max_gain;
     double rx_freq;
     double tx_freq;
+    std::string calibration_mode;
 };
 
 /*! Interface for daughterboards which support being plugged into a X400 motherboard.
@@ -37,6 +36,11 @@ public:
     //! Returns the parameters required to generate a suitable loopback tone at
     //! tone_freq to perform ADC self cal.
     virtual adc_self_cal_params_t get_adc_self_cal_params(double tone_freq) = 0;
+
+    //! Ask the dboard to search for a gain within the current device settings
+    //! that is suitable for ADC self calibration.
+    //! Returns true, if a suitable gain was found, false otherwise.
+    virtual bool select_adc_self_cal_gain(size_t chan) = 0;
 };
 
 }}} // namespace uhd::usrp::x400
