@@ -13,6 +13,7 @@ import copy
 from random import choice
 from string import ascii_letters, digits
 from multiprocessing import Process
+from multiprocessing import RLock
 import threading
 import sys
 from gevent.server import StreamServer
@@ -30,9 +31,10 @@ from usrp_mpm.sys_utils import watchdog
 from usrp_mpm.sys_utils import net
 
 TIMEOUT_INTERVAL = 5.0 # Seconds before claim expires (default value)
+LOCK_ACQ_TIMEOUT = 1 # Seconds to wait for acquiring shared lock (default value)
 TOKEN_LEN = 16 # Length of the token string
 # Compatibility number for MPM
-MPM_COMPAT_NUM = (4, 2)
+MPM_COMPAT_NUM = (4, 4)
 
 def no_claim(func):
     " Decorator for functions that require no token check "
@@ -62,6 +64,7 @@ class MPMServer(RPCServer):
                        MPM_COMPAT_NUM[0], MPM_COMPAT_NUM[1])
         self._state = state
         self._timer = Greenlet()
+        self._timer_lock = RLock()
         # Setting this to True will disable an unclaim on timeout. Use with
         # care, and make sure to set it to False again when finished.
         self._disable_timeouts = False
@@ -263,6 +266,18 @@ class MPMServer(RPCServer):
                 len(token) == TOKEN_LEN and \
                 self._state.claim_token.value == token
 
+    def _acquire_or_throw(self, lock, error_msg, timeout=LOCK_ACQ_TIMEOUT):
+        """
+        Attempt to acquire a shared lock. If the timeout is exceeded, then
+        throw an error.
+        """
+        acquired = lock.acquire(timeout=LOCK_ACQ_TIMEOUT)
+        if not acquired:
+            self.log.error(error_msg)
+            self._last_error = error_msg
+            raise RuntimeError("RPC Server Lock Acquire Timeout: " + error_msg)
+
+
     def claim(self, session_id):
         """Claim device
 
@@ -272,7 +287,8 @@ class MPMServer(RPCServer):
 
         Will return a token on success, or raise an Exception on failure.
         """
-        self._state.lock.acquire()
+        error_msg = "Claim timed out acquiring the shared state lock after 1 second."
+        self._acquire_or_throw(self._state.lock, error_msg)
         if self._state.claim_status.value:
             error_msg = \
                 "Someone tried to claim this device again (From: {})".format(
@@ -294,19 +310,20 @@ class MPMServer(RPCServer):
         self.periph_manager.claim()
         if self.periph_manager.clear_rpc_registry_on_unclaim:
             self._init_rpc_calls(self.periph_manager)
+        token_val = self._state.claim_token.value
         self._state.lock.release()
         self.session_id = session_id + " ({})".format(self.client_host)
         self._reset_timer()
         self.log.debug(
             "giving token: %s to host: %s",
-            self._state.claim_token.value,
+            token_val,
             self.client_host
         )
         if _is_connection_local(self.client_host):
             self.periph_manager.set_connection_type("local")
         else:
             self.periph_manager.set_connection_type("remote")
-        return self._state.claim_token.value
+        return token_val
 
     def reclaim(self, token):
         """
@@ -315,7 +332,8 @@ class MPMServer(RPCServer):
         claimed at all.
         """
         if self._state.claim_status.value:
-            self._state.lock.acquire()
+            error_msg = "Reclaim timed out acquiring the shared state lock after 1 second."
+            self._acquire_or_throw(self._state.lock, error_msg)
             if self._check_token_valid(token):
                 self._state.lock.release()
                 self.log.debug("reclaimed from: %s", self.client_host)
@@ -339,12 +357,16 @@ class MPMServer(RPCServer):
 
         Resets and deinitalizes the periph manager as well.
         """
-        self._state.lock.acquire()
+        error_msg = "Unclaim timed out acquiring the shared state lock after 1 second."
+        self._acquire_or_throw(self._state.lock, error_msg)
         self.log.debug(
             "Deinitializing device and releasing claim on session `{}'"
             .format(self.session_id))
         # Disable unclaim timer, we're now finished with reclaim loops.
+        error_msg = "Unclaim timed out acquiring the timer lock after 1 second."
+        self._acquire_or_throw(self._timer_lock, error_msg)
         self._timer.kill(block=False)
+        self._timer_lock.release()
         # We might need to clear the method registry
         if self.periph_manager.clear_rpc_registry_on_unclaim:
             self.clear_method_registry()
@@ -394,8 +416,11 @@ class MPMServer(RPCServer):
         Reset unclaim timer. After calling this, call this function again
         within 'timeout' seconds to avoid a timeout event.
         """
+        error_msg = "Reset Timer timed out acquiring the timer lock after 1 second."
+        self._acquire_or_throw(self._timer_lock, error_msg)
         self._timer.kill()
         self._timer = spawn_later(self._timeout_interval, self._timeout_event)
+        self._timer_lock.release()
 
     @contextmanager
     def _timeout_disabler(self):
@@ -625,7 +650,7 @@ def spawn_rpc_process(state, udp_port, default_args):
     """
     proc = Process(
         target=_rpc_server_process,
-        args=[udp_port, state, default_args],
+        args=[state, udp_port, default_args],
     )
     proc.start()
     return proc
